#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdint.h>
#include <unistd.h>
#include <math.h>
#include <time.h>

#define PROGRAM "gen_test"
#define VERSION "0.1"

int verbose_g;

void panic(const char *str, ...)
{
    va_list args;

    fprintf(stderr, "%s: ",PROGRAM);
    va_start(args, str);
    vfprintf(stderr, str, args);
    va_end(args);
    fputc('\n', stderr);
    exit(1);
}


int main(int argc, char *argv[])
{
    FILE *fp;
    char *destname;
    int c;
    int length;
    int seed;

    /* defaults */
    verbose_g=0;
    length=5000;
    seed=time(0);

    /*
     * scan for valid options
     */
    while (EOF!=(c=getopt (argc, argv, "l:vVh"))) {
        switch (c) {
	
	/* a missing parameter */
	case ':':
	/* an illegal option */
	case '?':
	    exit (1);

	/* print version */
	case 'v':
	    verbose_g=1;
	    break;

	/* print version */
	case 'V':
	    fprintf (stdout, PROGRAM " " VERSION "\n");
	    exit(0);

	/* print help */
	case 'h':
	    fprintf(stderr,
PROGRAM " " VERSION ": generate test data\n"
"Copyright (c) 2008 Daniel Kahlin <daniel@kahlin.net>\n"
"Written by Daniel Kahlin <daniel@kahlin.net>\n"
"\n"
"usage: " PROGRAM "[-v][-h][-V] <name>...\n"
"\n"
"Valid options:\n"
"    -v              be verbose\n"
"    -h              displays this help text\n"
"    -V              output program version\n"
	    );
	    exit(0);

	/* set length */
	case 'l':
	    length=atoi(optarg);
	    break;

	/* set seed */
	case 's':
	    seed=atoi(optarg);
	    break;

	/* default behavior */
	default:
	    break;
	}
    }

    /*
     * optind now points at the first non option argument
     * we expect one more argument inname
     */
    int num_names=argc-optind;
    if (num_names < 1)
	panic("too few arguments");

    destname=argv[optind];

    /* read source data */
    fp=fopen(destname,"wb");
    if (!fp)
	panic("couldn't open file for writing");

    /* load addr */
    fputc(0x01, fp);
    fputc(0x08, fp);

    srandom(seed);
    uint8_t a=0, b=0;
    int i;
    for (i=0; i < length-2; i++) {
	uint8_t d = random();
	fputc(d,fp);
	a+=d;
	b+=a;
    }
    fputc(a,fp);
    fputc(b,fp);
    fclose(fp);

    exit(0);
}
/* eof */
