;**************************************************************************
;*
;* FILE  ld2_1541.asm
;* Copyright (c) 2008, 2009 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: ld2_1541.asm,v 1.123 2009-04-11 20:44:39 tlr Exp $
;*
;* DESCRIPTION
;*   Loader II drive side code
;*
;******
	processor 6502
	include	"macros.i"
	INFO	"--- start of file ld2_1541.asm ---"		
	
	subroutine	reader_code
;**************************************************************************
;*
;* NAME  reader_code
;*
;* DESCRIPTION
;*   Drive side reader code.  
;*
;******
FAST_STEP_TIMER		equ	$1c ; 8 ms
NORMAL_STEP_TIMER	equ	$3a ; 15 ms
TIMER_10MS		equ	$28 ; 10 ms
SPIN_UP_TIME		equ	50 ; * 10 ms = 500 ms
INITIAL_TRACK		equ	18
MAX_TRACK		equ	40
MAX_SECTORS		equ	21

;BUF_LEN		equ	[325+10]/5 
BUF_LEN		equ	325/5	

block_buf5	equ	$0100-BUF_LEN
block_buf2	equ	block_buf5-BUF_LEN
block_buf1	equ	block_buf2-BUF_LEN
block_buf3	equ	$0100
block_buf4	equ	block_buf3+BUF_LEN
rtab		equ	block_buf4+BUF_LEN
nyb_tab		equ	$0200
gcr_tab		equ	$0700

	
	SHOWRANGE "block_buf1",block_buf1,block_buf1+BUF_LEN
	SHOWRANGE "block_buf2",block_buf2,block_buf2+BUF_LEN
	SHOWRANGE "block_buf3",block_buf3,block_buf3+BUF_LEN
	SHOWRANGE "block_buf4",block_buf4,block_buf4+BUF_LEN
	SHOWRANGE "block_buf5",block_buf5,block_buf5+BUF_LEN
	SHOWRANGE "rtab",rtab,rtab+MAX_SECTORS
	SHOWRANGE "nyb_tab",nyb_tab,nyb_tab+$0100
	SHOWRANGE "gcr_tab",gcr_tab,gcr_tab+$0100

.track_zp	equ	$09
.shift_zp	equ	$0a
currhalftrk_zp	equ	$0b
.tmp_zp		equ	$0c
secmax_zp	equ	$0d
secleft_zp	equ	$0e
sectmp_zp	equ	$0f

	org	$0300
reader_code:
; initial jump
	jmp	reader_code_entry

;**************************************************************************
;*
;* NAME  main_loop
;*
;* DESCRIPTION
;*   receive commands and perform them
;*   Called in FDC-ish mode
;*
;*   command entry
;*     1-35 = track
;*     $80  = exit
;*     $81  = motor off
;*     $82  = disk change
;*     $83  = re-read
;*     $84  = seek
;*
;******
main_loop:
; reset stack (an rts will 'jmp main_loop' after this)
	ldx	#$fd
	txs

	jsr	.get_byte
	bmi	ml_cmd
; safe against large tracks
ml_read:
	jsr	seek_track

	jmp	read_and_send_full

ml_cmd:
; Acc>=$80
	cmp	#$80+NUM_CMDS	; cmd out of range?
	bcs	ml_fl1		; yes, fail!
	tay
	lda	cmd_tab_h-$80,y
	pha
	lda	cmd_tab_l-$80,y
	pha
	rts

cmd_tab_l:
	dc.b	<[cmd_exit-1]
	dc.b	<[cmd_motor_off-1]
	dc.b	<[cmd_disk_change-1]
	dc.b	<[cmd_re_read-1]
	dc.b	<[cmd_seek-1]
cmd_tab_h:
	dc.b	>[cmd_exit-1]
	dc.b	>[cmd_motor_off-1]
	dc.b	>[cmd_disk_change-1]
	dc.b	>[cmd_re_read-1]
	dc.b	>[cmd_seek-1]
NUM_CMDS	equ	.-cmd_tab_h

ml_fl1:
	lda	#$20	; $20 Track or Command out of range
	jmp	error_state

;**************************************************************************
;*
;* NAME  cmd_exit
;*
;* DESCRIPTION
;*   Exit to DOS (Reset vector)
;*
;******
cmd_exit:
	jmp	($fffc)

;**************************************************************************
;*
;* NAME  motor_on, motor_off, cmd_motor_off
;*
;* DESCRIPTION
;*   Start/stop motor.  Wait for spin-up time when going from off to on.
;*
;******
motor_on:
	lda	#%00000100	; motor on?
	bit	$1c00
	bne	mo_ex1		; yes, skip out
	ora	$1c00		; motor on
	sta	$1c00
	ldy	#SPIN_UP_TIME
mo_lp1:
	lda	#$80 | TIMER_10MS ; ~10 ms
	jsr	delay
	dey
	bne	mo_lp1
mo_ex1:
	rts

cmd_motor_off:
motor_off:
	lda	$1c00
	and	#%11110011	; LED off, motor off
	sta	$1c00
	rts

;**************************************************************************
;*
;* NAME  cmd_disk_change
;*
;* DESCRIPTION
;*   Seek track 18, and wait until syncs disappear, and then reappear
;*
;******
cmd_disk_change:
	lda	#18
	jsr	seek_track

	lda	$1c00
	and	#%11110111	; LED off
	sta	$1c00
	lda	#$80 | $7f	; ~32 ms
cds_lp1:
	sta	$1805
cds_lp2:
	bit	$1c00		; Sync found?
	bpl	cds_lp1	; yes, reset timer and find next
	bit	$1805		; timeout?
	bmi	cds_lp2	; no, check again

	lda	#0
	jsr	.put_byte	; disk pulled
cds_lp3:
	bit	$1c00		; Sync found?
	bmi	cds_lp3	; no, wait for sync

	lda	#1
	jmp	.put_byte	; disk inserted again
; 	rts			

;**************************************************************************
;*
;* NAME  cmd_re_read
;*
;* DESCRIPTION
;*   read and send selected sectors
;*
;*   00: cmd re-read ($83)
;*   01: sector mask  7-0
;*   02: sector mask 15-8
;*   03: sector mask 20-16
;*
;******
cmd_re_read:
	ldy	#0
	sty	secleft_zp	; for safety
crr_lp1:
	tya
	and	#7
	bne	crr_skp1
	jsr	.get_byte	; fetch to .shift_zp
crr_skp1:
	lsr	.shift_zp
	bcc	crr_skp2
; mark sectors with 1 bits as unread
	lda	#0
	sta	rtab,y
	inc	secleft_zp	
crr_skp2:
	iny
	cpy	secmax_zp
	bne	crr_lp1

	beq	read_and_send	; always taken

;**************************************************************************
;*
;* NAME  read_and_send_full
;*
;* DESCRIPTION
;*   read and send full track
;*
;******
read_and_send_full:

; wipe the rtab (i.e seen sectors)
	ldx	secmax_zp
	lda	#0
rsf_lp1:
	sta	rtab-1,x
	dex
	bne	rsf_lp1

	; fall thru
;**************************************************************************
;*
;* NAME  read_and_send
;*
;* DESCRIPTION
;*   read and send sectors
;*
;******
read_and_send:
ras_lp1:
; Find Header Block
; HDR -> 01010 01001 ($08)
; CHK -> xxxxx yyyyy
; SEC -> 0101x yyyyy
; TRK -> 0101x yyyyy or 10010 yyyyy
; IDL -> xxxxx yyyyy
; IDH -> xxxxx yyyyy
; OF1 -> 01010 10101 ($0f)
; OF2 -> 01010 10101 ($0f)
; GAP -> 01111 01111 ($55 * 8)

	ldx	#100
ras_lp2:
	dex
	beq	ras_fl1
	jsr	wait_sync	; wait for sync
	bvc	.
	clv
	lda	$1c01		; HDR bit 9-2
	cmp	#%01010010	; GCR for $08 (Header Block)?
	bne	ras_lp2		; no, try again
	bvc	.
	clv
	lda	$1c01		; HDR bit 1-0 | CHK bit 9-4
	and	#%11000000
	cmp	#%01000000
	bne	ras_lp2
	bvc	.
	clv
	lda	$1c01		; CHK bit 3-0 | SEC bit 9-6
	and	#%00001111
	cmp	#%00000101	; 0101x = $00 or $01
	bne	ras_lp2
	bvc	.
	clv
	lda	$1c01		; SEC bit 5-0 | TRK bit 9-8
	lsr
	lsr
	sta	sectmp_zp	; = SEC bit 5-0
	bvc	.
	clv
	lda	$1c01		; TRK bit 7-0
	and	#%11000000
	cmp	#%01000000	; xx01x = $00-$03 (or $0a-$0b)
	bne	ras_lp2

; Header Block found, check SEC number
	lda	sectmp_zp
	and	#%00011111
	tay
	lda	sectmp_zp
	lsr
	and	#%00010000
	ora	$f8c0,y		; GCR -> low nybble ($ff for invalid)
; Acc=sector (or $ff for invalid)
	cmp	secmax_zp	; larger than sectors on track (or $ff)?
	bcs	ras_lp2		; yes, try again
	tay
	lda	rtab,y		; already visited?
	bne	ras_lp2		; yes, seek next
	lda	#1
	sta	rtab,y		; mark as visited
; Unique sector header found! Y=Sector
; Store it as the first byte transferred to the DTV.
	lda	gcr_tab,y
	sta	block_buf1+BUF_LEN-1

; Read Data Block
; HDR -> 01010 10101 ($07)
; DAT -> xxxxx yyyyy ($xy * 256) = 320 GCR bytes
; CHK -> xxxxx yyyyy ($xy)
; OF1 -> 01010 01010 ($00)
; OF2 -> 01010 01010 ($00)
; GAP -> 01111 01111 ($55 * n)

	jsr	wait_sync	; wait for sync
	bvc	.
	clv
	lda	$1c01		; HDR bit 9-2
	cmp	#%01010101	; GCR for $07 (Data Block)?
	bne	ras_fl2		; no, fail

	jsr	read_sector
	jsr	send_sector
		
	dec	secleft_zp	
	bne	ras_lp1

	rts
;	jmp	main_loop

ras_fl1:		; $02 Header Block Not Found
	lda	#$02
	dc.b	$2c
ras_fl2:		; $04 Data Block Not Found
	lda	#$04
	; fall thru
;**************************************************************************
;*
;* NAME  error_state
;*
;* DESCRIPTION
;*   Signal error on the IEC bus and send the error code.
;*
;******
error_state:
	ldx	#%0001010	; CLK=0, DATA=0
	stx	$1800
	ldx	#%0001000	; CLK=0, DATA=1
	jsr	.put_byte_err
 	jmp	main_loop	; to allow for stack reset

;**************************************************************************
;*
;* NAME  read_sector
;*
;* DESCRIPTION
;*   read sector data + checksum byte from disk to the buffers
;*
;******
read_sector:	
	ldx	#BUF_LEN-1
	START_SAMEPAGE "read_sector"
.rs_lp1:
	bvc	.
	clv
	ldy	$1c01
	lda	gcr_tab,y
	sta	block_buf2,x
	bvc	.
	clv
	ldy	$1c01
	lda	gcr_tab,y
	sta	block_buf3,x
	bvc	.
	clv
	ldy	$1c01
	lda	gcr_tab,y
	sta	block_buf4,x
	bvc	.
	clv
	ldy	$1c01
	lda	gcr_tab,y
	sta	block_buf5,x
	dex
	bmi	.rs_ex1
	bvc	.
	clv
	ldy	$1c01
	lda	gcr_tab,y
	sta	block_buf1,x
	jmp	.rs_lp1
	END_SAMEPAGE
.rs_ex1:
	rts

;**************************************************************************
;*
;* NAME  send_sector
;*
;* DESCRIPTION
;*   Send sector data to the DTV over the IEC bus
;*
;******
send_sector:
	lda	#$0f
	sta	.tmp_zp
	lda	#%00000100
	sta	$1800		; CLK=1, DATA=1
	START_SAMEPAGE "send_sector1"
ssc_lp1:
	bit	$1800		; CLK=0?
	beq	ssc_lp1		; no, loop
	lsr
	sta	$1800		; DATA=0
	ldx	#2
ssc_lp2:
	dex
	bne	ssc_lp2
	END_SAMEPAGE
	stx	$1800		; CLK=1, DATA=1
	nop
	nop
	sta	$1800		; DATA=0
	bit	$ea
	stx	$1800		; DATA=1
; Acc= (DATA=0,	CLK=1)

	ldx	#BUF_LEN-1	; 2
ssc_lp3:
	sta	$1800		; 4	; sync

	lda	block_buf1,x	; 4
	tay			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	lda	nyb_tab,y	; 4
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4

	lda	block_buf2,x	; 4
	tay			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	lda	nyb_tab,y	; 4
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4

	lda	block_buf3,x	; 4
	tay			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	lda	nyb_tab,y	; 4
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4

	lda	block_buf4,x	; 4
	tay			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	lda	nyb_tab,y	; 4
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4

	lda	block_buf5,x	; 4
	tay			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4
	lda	nyb_tab,y	; 4
	sta	$1800		; 4
	asl			; 2
	and	#$0f		; 2
	sta	$1800		; 4

	eor	.tmp_zp		; 3
	dex			; 2
	START_SAMEPAGE "send_sector2"
	bmi	ssc_ex1		; 2 (3)
	jmp	ssc_lp3		; 3
ssc_ex1:
	END_SAMEPAGE
	ldx	#%00000010
	stx	$1800		; CLK=1, DATA=0
	rts


;**************************************************************************
;*
;* NAME  cmd_seek
;*
;* DESCRIPTION
;*   seek track
;*
;*   00: cmd seek ($84)
;*   01: track
;*
;******
cmd_seek:
	jsr	.get_byte

	; fall thru
;**************************************************************************
;*
;* NAME  seek_track
;*
;* DESCRIPTION
;*   "Seek" track "Acc" and setup density and number of sectors.
;*   Start motor if it was not already on.
;*
;******
seek_track:
	cmp	#MAX_TRACK+1	; track out of range?
	bcs	st_fl1		; yes, fail!

	sta	.track_zp

	jsr	motor_on

st_lp1:
	lda	.track_zp
	asl			; Count half-tracks
	cmp	currhalftrk_zp
	beq	st_ex1		; currhalftrk_zp == .track_zp*2
	bcc	st_skp1		; currhalftrk_zp > .track_zp*2
; currhalftrk_zp < .track_zp*2
	jsr	head_in
	inc	currhalftrk_zp
	jmp	st_lp1
st_skp1:
	jsr	head_out
	dec	currhalftrk_zp
	jmp	st_lp1
st_ex1:

; set up number of sectors on track, and density
start_num_dens:
	lda	$1c00		; set density
	ldy	.track_zp
	ldx	#21
	ora	#%01100000 | LED_STATE
	cpy	#17+1
	bcc	st_done		; 1-17
	ldx	#19
;	sec
	sbc	#%00100000
	cpy	#24+1
	bcc	st_done		; 17-24
	dex
;	sec
	sbc	#%00100000
	cpy	#30+1
	bcc	st_done		; 25-30
	dex
;	sec
	sbc	#%00100000
st_done:
	stx	secleft_zp
	stx	secmax_zp
	sta	$1c00
	rts
st_fl1:	
	jmp	ml_fl1		; track number out of range

LED_STATE	equ	%00001000
end_num_dens:
	SHOWRANGE "num_dens", start_num_dens, end_num_dens
	
; step head outwards (= lower track num)
head_out:
	lda	#$ff
	dc.b	$2c
; step head inwards (= higher track num)
head_in:
	lda	#$01
head_common:
	clc
	adc	$1c00
	and	#%00000011
	sta	.tmp_zp
	lda	$1c00	
	and	#%11110100	; mask + led off
	ora	.tmp_zp
	sta	$1c00
	lda	#$80 | FAST_STEP_TIMER
delay:
	sta	$1805
dl_lp1:
	bit	$1805
	bmi	dl_lp1
	rts

;**************************************************************************
;*
;* NAME  .put_byte, .put_byte_err
;*
;* DESCRIPTION
;*   Put byte to DTV using 1-bit protocol
;*
;******
.put_byte:
	ldx	#%00000000	; CLK=1, DATA=1
.put_byte_err:
	stx	$1800
	sta	.shift_zp

	ldx	#4
	lda	#%00000001
	START_SAMEPAGE	"put_byte"
.pb_lp1:
	bit	$1800		; DATA=0?
	beq	.pb_lp1		; no, wait more
.pb_lp2:
	ror	.shift_zp
	rol
	rol
	and	#%00000010
	sta	$1800
	lda	#%00000100
.pb_lp3:
	bit	$1800		; CLK=0?
	beq	.pb_lp3		; no, wait
	ror	.shift_zp
	rol
	rol
	and	#%00000010
	sta	$1800
	lda	#%00000100
.pb_lp4:
	bit	$1800		; CLK=1?
	bne	.pb_lp4		; no, wait
	dex
	bne	.pb_lp2
	END_SAMEPAGE
	lda	#%00000010	; CLK=1, DATA=0
	sta	$1800
	rts

;**************************************************************************
;*
;* NAME  .get_byte
;*
;* DESCRIPTION
;*   Get byte from DTV using 1-bit protocol
;*
;******
.get_byte:
	ldx	#4
	stx	$1800		; CLK=1, DATA=1
	START_SAMEPAGE	"get_byte"
.gb_lp1:
	lda	#%00000100
.gb_lp2:
	bit	$1800		; CLK=0?
	beq	.gb_lp2		; no, wait
	lda	$1800
	lsr
	ror	.shift_zp
	lda	#%00000100
.gb_lp3:
	bit	$1800		; CLK=1?
	bne	.gb_lp3		; no, wait
	lda	$1800
	lsr
	ror	.shift_zp
	dex
	bne	.gb_lp1
	END_SAMEPAGE
	lda	#%00000010	; CLK=1, DATA=0
	sta	$1800
	lda	.shift_zp
	rts	
	
;**************************************************************************
;*
;* NAME  wait_sync
;*
;* DESCRIPTION
;*   Clone of $f556, but with custom routine on timeout.
;*
;******
wait_sync:
	lda	#$80 | $50	; 20 ms
	sta	$1805
ws_lp1:
	bit	$1805		; timeout?
	bpl	ws_fl1		; yes, fail exit
	bit	$1c00		; Sync found?
	bmi	ws_lp1		; no, try again
	lda	$1c01
	clv			; flush byte
	rts
ws_fl1:
	lda	#$03		; $03 SYNC not found
	jmp	error_state
	
reader_code_stop:


;**************************************************************************
;*
;* NAME  reader_code_entry
;*
;* DESCRIPTION
;*   Drive code entry point
;*   Called in IP mode, interrupts disabled
;*   initial (disposable) initialization code
;*
;******
reader_code_entry:
; custom seek
	lda	#%11101110
	sta	$1c0c

; handle if we are positioned on a half track
	lda	$1c00
	lsr
	bcc	wce_skp1
	jsr	head_in
wce_skp1:
; step in (higher track), then out (lower track) to settle head
; This is because some drive mechanisms will feel bad when going
; directly from step 10 -> step 00 (i.e track 18 -> reset routine)	
	jsr	head_in		
 	jsr	head_out

; scan for valid sector headers to determine which track we are actually
; on!

	lda	#$3f
	sta	$4b
rce_lp1:
	lda	$4b
	and	#7
	bne	rce_skp1
; change the density every 8 retries
	lda	$1c00
	clc
	adc	#%00100000
;	and	#%01111111	; sync bit doesn't matter.
	sta	$1c00
rce_skp1:

	lda	#$80 | $50	; 20 ms
	sta	$1805
rce_lp2:
	bit	$1805		; timeout?
	bpl	rce_fl1		; yes, fail exit
	bit	$1c00		; Sync found?
	bmi	rce_lp2		; no, try again
	lda	$1c01
	clv			; flush byte
	ldx	#0
rce_lp3:
	bvc	.
	clv
	lda	$1c01
	sta	$24,x
	inx
	cpx	#8
	bne	rce_lp3
	lda	$24
	cmp	#$52
	bne	rce_fl1
	jsr	$f497		; decode
	lda	$16
	eor	$17
	eor	$18
	eor	$19
	eor	$1a
	beq	rce_found
rce_fl1:
	dec	$4b
	bpl	rce_lp1

; failed, go to error state
	jmp	ras_fl1		; $02 Header Block Not Found
	
rce_found:
	lda	$18
	asl
	sta	currhalftrk_zp
	lda	#INITIAL_TRACK
	jsr	seek_track
	; fall thru

;**************************************************************************
;*
;* NAME  generate_tables
;*
;* DESCRIPTION
;*   gcr_tab:
;*     Look up table for raw GCR bytes to arrange bits to arrive at the
;*     DTV in linear order, non-inverted.  High nybbles are renumbered for
;*     look up in nyb_tab.
;*
;*   nyb_tab:
;*     High nybble look up table for 1541 -> DTV send.  
;*
;******
GENTAB_COPYDOWN	equ	0
GENTAB_TARGET	equ $0146
start_gentabtotal:
generate_tables:
	if	GENTAB_COPYDOWN
	ldx	#GENTAB_LEN
gt_lp1:
	lda	start_gentab-1,x
	sta	GENTAB_TARGET-1,x
	dex
	bne	gt_lp1
	jmp	GENTAB_TARGET
	endif
start_gentab:
; generate nyb_tab ($0200-$02ff)
; generate gcr_tab ($0700-$07ff)
	if	!GENTAB_COPYDOWN
	ldy	#0
	endif
gt_lp2:
	sty	.tmp_zp
	tya
	lsr
	lsr
	lsr
	lsr
	sta	nyb_tab,y
	lda	#0
	ldx	#8-1
gt_lp3:
	lsr	.tmp_zp
	bcs	gt_skp1
	ora	maplut,x
gt_skp1:
	dex
	bpl	gt_lp3
	sta	gcr_tab,y
	iny
	bne	gt_lp2
; X=$ff
; tables initialized, set stack and start the main code
	txs
	lda	#>[main_loop-1]
	pha
	if	<[main_loop-1] == $ff
	txa
	else
	lda	#<[main_loop-1]
	endif
	pha
	rts		; jmp	main_loop

	if	GENTAB_COPYDOWN
maplut	equ	.-[start_gentab-GENTAB_TARGET]
	else
maplut:
	endif
	dc.b	%00000010	; bit 7
	dc.b	%00001000	; bit 6
	dc.b	%00000001	; bit 5
	dc.b	%00000100	; bit 4
	dc.b	%00100000	; bit 3
	dc.b	%10000000	; bit 2
	dc.b	%00010000	; bit 1
	dc.b	%01000000	; bit 0
end_gentab:
GENTAB_LEN	equ	end_gentab-start_gentab
	SHOWRANGE "gentab",start_gentab,end_gentab
end_gentabtotal:
	SHOWRANGE "gentabtotal",start_gentabtotal,end_gentabtotal


free1_start	equ	reader_code_stop

free1_end	equ	gcr_tab
	
free2_start:
free2_end	equ	gcr_tab

reader_code_end:
READER_CODE_LEN	equ	.-reader_code
	SHOWRANGE "reader_code",reader_code,reader_code_stop
	if [rtab+MAX_SECTORS] < nyb_tab
	SHOWRANGE "free",[rtab+MAX_SECTORS], nyb_tab 
	endif
	if free1_start < free1_end
	SHOWRANGE "free (max)",free1_start, free1_end 
	endif
	if free2_start < free2_end
	SHOWRANGE "free (no overlap)",free2_start, free2_end 
	endif

; eof
