;**************************************************************************
;*
;* FILE  ld2_install.asm
;* Copyright (c) 2008, 2009 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: ld2_install.asm,v 1.96 2009-03-29 16:25:32 tlr Exp $
;*
;* DESCRIPTION
;*   Loader II DTV side install
;*
;*   frequencies: 985250 Hz (PAL), 1022730 (NTSC) and 1000000 MHz (1541)
;*
;*   PAL: 
;*     (985250/1000000)*68 = 66.99700
;*     i.e 67 C64/DTV cycles per 68 drive cycles
;*
;*   NTSC:
;*     (1022730/1000000)*44 = 45.00012 
;*     i.e 45 C64/DTV cycles per 44 drive cycles
;*
;******
	processor 6502
	include	"macros.i"
	INFO	"--- start of file ld2_install.asm ---"		

	seg.u	inst_zp
	org	$40
in_offs_zp:
	ds.b	1
in_bank_zp:
	ds.b	1
in_bank2_zp:
	ds.b	1
in_shift_zp:
	ds.b	1

	seg	code
installer_start:
;**************************************************************************
;*
;* NAME  init_drive
;*
;* DESCRIPTION
;*   Install initial boot code in drive
;*
;******
MAX_M_W	equ	$20
NUM_M_W_CHUNKS	equ	[BOOT_CODE_LEN+MAX_M_W-1]/MAX_M_W
	INFO	"num M-W chunks",NUM_M_W_CHUNKS

init_drive:

	ldx	#NUM_M_W_CHUNKS
	ldy	#0
id_lp1:	
	lda	#"W"
	jsr	listen_send_m
	lda	tab_low-1,x
	jsr	$ffa8
	lda	tab_high-1,x
	jsr	$ffa8
	lda	#MAX_M_W
	jsr	$ffa8
	tya
	clc
	adc	#MAX_M_W
	sta	in_offs_zp
id_lp2:
	lda	boot_code_st,y
	jsr	$ffa8
	iny
	cpy	in_offs_zp
	bne	id_lp2
	
	jsr	$ffae		; unlsn

	dex
	bne	id_lp1
	
; run the code
	lda	#"E"
	jsr	listen_send_m
	lda	#<boot_code_entry
	jsr	$ffa8
	lda	#>boot_code_entry
	jsr	$ffa8
	jsr	$ffae		; unlsn

; synchronize the loader
 	sei
	lda	$dd00
	and	#%00000011
	sta	in_bank_zp
	tay
	asl
	asl
	asl
	asl
	sta	in_bank2_zp
; Y=in_bank_zp

; wait for drive to signal busy so we know it's active
id_lp3:
	bit	$dd00		; DATA=1, CLK=0?
	bpl	id_lp3		; no, DATA=0, loop
	bvs	id_lp3		; no, CLK=1, loop
	tya
	ora	#%00100000	; DATA=0
	sta	$dd00
id_lp4:
	bit	$dd00		; CLK=1?
	bvc	id_lp4		; no, CLK=0, loop
	sty	$dd00		; DATA=1

.OFFS	equ	[-READER_CODE_LEN]&$ff
.ENDPAGE equ	>[reader_code_st+READER_CODE_LEN]
	ldy	#.OFFS
id_lp5:
id_sm2:
	lda	reader_code_st-.OFFS,y
	jsr	put_byte_boot
	iny
	bne	id_lp5
	inc	id_sm2+2
	lda	id_sm2+2
	cmp	#.ENDPAGE
	bne	id_lp5
	echo	"copy",reader_code_st,reader_code_st+READER_CODE_LEN,.OFFS,.ENDPAGE

	cli
	rts

tab_low:
	dc.b	<[boot_code+2*MAX_M_W]
	dc.b	<[boot_code+1*MAX_M_W]
	dc.b	<[boot_code+0*MAX_M_W]
tab_high:
	dc.b	>[boot_code+2*MAX_M_W]
	dc.b	>[boot_code+1*MAX_M_W]
	dc.b	>[boot_code+0*MAX_M_W]

listen_send_m:
	pha
	lda	$ba		; drive #
	jsr	$ffb1		; listen
	lda	#$6f
	jsr	$ff93		; lstsec
	lda	#"M"
	jsr	$ffa8
	lda	#"-"
	jsr	$ffa8
	pla
	jmp	$ffa8

	subroutine	put_byte
;**************************************************************************
;*
;* NAME  put_byte
;*
;* DESCRIPTION
;*   Send byte to drive using 2-bit protocol
;*
;******
put_byte_boot:
	sta	in_shift_zp
.pb_lp1:
	bit	$dd00		; DATA=0?
	bpl	.pb_lp1		; yes, drive busy
	lda	in_bank_zp
	ora	#%00010000
	sta	$dd00		; CLK=0
	ldx	#4		; 2
	START_SAMEPAGE	"put_byte_boot"
.pb_lp2:
	lda	in_bank2_zp	; 3
	lsr	in_shift_zp	; 5
	ror			; 2
	lsr	in_shift_zp	; 5
	ror			; 2
	lsr			; 2
	lsr			; 2
	sta	$dd00		; 4
	nop			; 2
	dex			; 2
	bne	.pb_lp2		; 3(2)	= 32 (21)
	ldx	#2
.pb_lp3:
	dex
	bne	.pb_lp3
	END_SAMEPAGE
	lda	in_bank_zp
	sta	$dd00
	rts

	subroutine	boot_code
;**************************************************************************
;*
;* NAME  boot_code
;*
;* DESCRIPTION
;*   Drive side boot code.  
;*   Receive real code using 2-bit protocol.
;*
;******
boot_code_st:
	rorg	reader_code_end
;	rorg	$0146
.shift_zp	equ	$0a
	
boot_code:

get_byte_boot:
	lda	#%00000100
	sta	$1800		; CLK=1, DATA=1
	START_SAMEPAGE	"get_byte_boot"
.gb_lp1:
	bit	$1800		; CLK=0?
	beq	.gb_lp1		; no, loop
; 	ldx	#4
	tax
.gb_lp3:
	dex
	bne	.gb_lp3
; 	ldx	#4
	tax
.gb_lp4:
	lda	$1800		; 4
	lsr			; 2
	php			; 3
	lsr			; 2
	lsr			; 2
	ror	.shift_zp	; 5
	plp			; 4
	ror	.shift_zp	; 5
	dex			; 2
	bne	.gb_lp4		; 3 (2) = 32 (31)
	END_SAMEPAGE
	lda	#%00000010
	sta	$1800		; CLK=1, DATA=0
	lda	.shift_zp
	rts
get_byte_boot_end:
	SHOWRANGE "get_byte_boot", get_byte_boot, get_byte_boot_end
		
boot_code_entry:
	sei
; signal that we have entered the boot code.
	lda	#%00001000
	sta	$1800		; CLK=0, DATA=1
; Spin up drive motor so it will be done by the time we have transferred
; the real code.
;	lda	#%00000100
	lsr
	ora	$1c00
	sta	$1c00		; Motor on

; Wait for the DTV to signal it is ready.
.bc_lp1:
	lda	$1800		; DATA=0?
	lsr
	bcc	.bc_lp1		; no, loop

.OFFS	equ	[-READER_CODE_LEN]&$ff
.ENDPAGE equ	>[reader_code+READER_CODE_LEN]
	ldy	#.OFFS
.bc_lp2:
	jsr	get_byte_boot
.bc_sm1:
	sta	reader_code-.OFFS,y
	iny
	bne	.bc_lp2
	inc	.bc_sm1+2
	lda	.bc_sm1+2
	cmp	#.ENDPAGE
	bne	.bc_lp2
	echo	"target",reader_code,reader_code+READER_CODE_LEN,.OFFS,.ENDPAGE
	jmp	reader_code
boot_code_end:
	rend
BOOT_CODE_LEN	equ	.-boot_code_st
	SHOWRANGE "boot_code",boot_code,boot_code_end
	

;**************************************************************************
;*
;* SECTION  external code
;*   
;******
reader_code_st:
	rorg	$0300
reader_code:
	incbin	"ld2_1541.prg",2
reader_code_end:
	rend
READER_CODE_LEN	equ	.-reader_code_st

installer_end:
	SHOWRANGE "installer",installer_start,installer_end
; eof
