;**************************************************************************
;*
;* FILE  loader2.asm
;* Copyright (c) 2008 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: loader2.asm,v 1.57 2009-03-22 12:56:44 tlr Exp $
;*
;* DESCRIPTION
;*   Fast DMA IRQ loader for the DTV / 1541 compatible.
;*
;******
	processor 6502

	include	"macros.i"
	INFO	"--- start of file loader2.asm ---"

	subroutine "loader2"

LOAD_ADDR2_SKIP	equ	0
LOAD_ADDR2	equ	1
LOAD_ADDR3_SKIP	equ	2
LOAD_ADDR3	equ	3
LOAD_RAW	equ	4

	ifconst LOADER2_TEST
; debugging options
DEBUG_SECTOR_ERRORS	equ	1
DEBUG_DISKCHANGE	equ	0
; Force files with 2 byte load addr onto $02xxxx 
LOAD_ADDR_BASE_PAGE	equ	$02 ; $02xxxx
; API features
HAVE_COUNT_ADDR		equ	1
	else
; debugging options
DEBUG_SECTOR_ERRORS	equ	0
DEBUG_DISKCHANGE	equ	0
; Force files with 2 byte load addr onto $00xxxx 
LOAD_ADDR_BASE_PAGE	equ	$00 ; $00xxxx
; API features
HAVE_COUNT_ADDR		equ	0
	endif

	include	"ld2_zp_bss.asm"

	
	seg	code
loader_start:
;**************************************************************************
;*
;* SECTION  jump table
;*
;******
ld2_install:
	jmp	.install
ld2_uninstall:
	jmp	.uninstall
ld2_setnam:
 	jmp	.setnam		
ld2_setstart:
	jmp	.setstart
ld2_load:
	jmp	.load
ld2_silence:
 	jmp	.silence
ld2_diskchange:			
	jmp	.diskchange
	if	HAVE_COUNT_ADDR
ld2_getstart:
	jmp	.getstart
ld2_getend:
	jmp	.getend
	endif

	dc.b	"*** DMA LOADER II BY TLR'09 ***"

;**************************************************************************
;*
;* NAME  .set_mapping, .unset_mapping
;*
;* DESCRIPTION
;*
;******
.set_mapping:
	php
	sei
	sac	#$dd
	lda	#BUFFER_SEGMENT
	sac	#$00
	plp
	rts

.unset_mapping:
	php
	sei
	sac	#$dd
	lda	#$01
	sac	#$00
	plp
	rts

;**************************************************************************
;*
;* NAME  .install
;*
;* DESCRIPTION
;*
;******
.install:
	jsr	init_drive

	jsr	.set_mapping
	lda	#1
	sta	$d03f
; here the relevant segment has been set up.
	lda	#0
	sta	dir_cnt		; no dir in cache
	sta	last_track	; no track in cache
	jsr	.unset_mapping
	rts

;**************************************************************************
;*
;* NAME  .uninstall
;*
;* DESCRIPTION
;*
;******
.uninstall:
; restore segment mapping (if it was set by mistake)
	jsr	.unset_mapping
; un-install drive code
	lda	#$80		; un-install
	jmp	put_byte
;	rts

;**************************************************************************
;*
;* NAME  .silence
;*
;* DESCRIPTION
;*
;******
.silence:
	lda	#$81		; motor_off
	jmp	put_byte
; 	rts			

	
;**************************************************************************
;*
;* NAME  .diskchange
;*
;* DESCRIPTION
;*
;******
.diskchange:
dc_lp1:
	lda	#$82		; diskchange
	jsr	put_byte
	if	DEBUG_DISKCHANGE
	inc	$d020
	endif
	jsr	get_byte
	if	DEBUG_DISKCHANGE
	inc	$d020
	endif
	jsr	get_byte
	if	DEBUG_DISKCHANGE
	dec	$d020
	dec	$d020
	endif
	jsr	.set_mapping
	lda	#0
	sta	dir_cnt		; invalidate dir cache
	sta	last_track	; invalidate track cache
	jsr	prepare_dma
	jsr	cache_directory
	jsr	find_file_in_dir
	bcs	dc_lp1		; file not found, loop
	jsr	.unset_mapping
	rts

;**************************************************************************
;*
;* NAME  .setnam
;*
;* DESCRIPTION
;*
;******
.setnam:
	sta	namlen_zp
	stx	namptr_zp
	sty	namptr_zp+1
	rts

;**************************************************************************
;*
;* NAME  .setstart
;*
;* DESCRIPTION
;*
;******
.setstart:
	sta	tptr_zp
	stx	tptr_zp+1
	sty	tptr_zp+2
	rts

	if	HAVE_COUNT_ADDR
;**************************************************************************
;*
;* NAME  .getstart
;*
;* DESCRIPTION
;*
;******
.getstart:
	lda	strt_zp
	ldx	strt_zp+1
	ldy	strt_zp+2
	rts

;**************************************************************************
;*
;* NAME  .getend
;*
;* DESCRIPTION
;*
;******
.getend:
	lda	tptr_zp
	ldx	tptr_zp+1
	ldy	tptr_zp+2
	rts
	endif

;**************************************************************************
;*
;* NAME  .load
;*
;* DESCRIPTION
;*
;******
.load:
	jsr	.set_mapping
	sta	load_mode
	jsr	prepare_dma
	lda	dir_cnt
	bne	ld_skp1
	jsr	cache_directory
ld_skp1:
	jsr	find_file_in_dir
	bcc	load_file_ts

; file not found, exit with C=1
	jsr	.unset_mapping
	rts

; file found, now load it...
load_file_ts:
	jsr	prepare_load_addr
	lda	#0
	sta	lptr_zp
	sta	loadaddr_flag_zp
	sta	last_flag_zp
lf_lp1:
	lda	track
	jsr	read_track_cached

lf_lp2:
	lda	sector
	clc
	adc	#>track_buffer
	sta	lptr_zp+1
; 	clc			
	adc	#>DMABLIT_OFFSET
	sta	$d321		; Src A MidSB

; calculate last byte + 1
	ldy	#0
	lda	(lptr_zp),y
	bne	lf_skp1
	inc	last_flag_zp
	iny
	lda	(lptr_zp),y
	tay
	iny
lf_skp1:
	sty	endoffs_zp
	
	ldy	offset
; handle load address
lf_lp3:
	lda	loadaddr_flag_zp
	bne	lf_skp2
	jsr	get_load_addr
	cpy	endoffs_zp
	bne	lf_lp3
	beq	lf_skp4
lf_skp2:

; Y=start of data within block
	tya
	sta	$d320		; Src A LSB
	eor	#$ff
; Acc=-start-1			
	sec
	adc	endoffs_zp
; Acc=-start-1 + endoffs_zp+1 => endoffs_zp-start (Real length)
	
	sta	$d338		; Len
	ldx	#%00001111
	stx	$d33a		; Start Transfer

	if	HAVE_COUNT_ADDR
	clc
	adc	tptr_zp
	sta	tptr_zp
	bcc	lf_skp3
	inc	tptr_zp+1
	bne	lf_skp3
	inc	tptr_zp+2
lf_skp3:
	endif
lf_lp4:
	lda	$d33f
	lsr
	bcs	lf_lp4

	lda	#%00001000	; Dest Continue
	sta	$d33f

lf_skp4:
	lda	last_flag_zp
	bne	lf_ex1
; follow T/S link
	ldy	#2
	sty	offset
	dey
	lda	(lptr_zp),y
	sta	sector
	dey
	lda	(lptr_zp),y		
	beq	lf_ex1
	sta	track
	cmp	track_zp
	beq	lf_lp2
	jmp	lf_lp1
lf_ex1:
	clc
	jsr	.unset_mapping
	rts

	
prepare_load_addr:
	lda	load_mode
	lsr
	tax
	lda	offs_tab,x
	sta	count_zp
	lda	#tptr_zp
	sta	gla_sm1+1
	rts

offs_tab:
	dc.b	2, 3, 0, 0

get_load_addr:
	lda	count_zp
	beq	gla_skp2
	lda	load_mode
	lsr
	bcc	gla_skp1

; Force files with 2 byte load addr onto $00xxxx 
	lda	#LOAD_ADDR_BASE_PAGE
	sta	tptr_zp+2
	lda	(lptr_zp),y
gla_sm1:
	sta	tptr_zp
	inc	gla_sm1+1
gla_skp1:
	iny
	dec	count_zp
	bne	gla_ex1

gla_skp2:
; setup address
	ldx	#3
gla_lp1:
	lda	tptr_zp-1,x
	sta	$d330-1,x
	if	HAVE_COUNT_ADDR
	sta	strt_zp-1,x
	endif
	dex
	bne	gla_lp1

; flag done
	inc	loadaddr_flag_zp
		
gla_ex1:
	rts

HAVE_SETUP_BLITTER_HOOK	equ	1
	mac	SETUP_BLITTER_HOOK
	if	[track_buffer_real>>16]
	lda	#track_buffer_real>>16
	sta	$d322		; Src A MSB
	endif
	lda	#$02
	sta	$d332		; Dest MSB
	lda	#1
	sta	$d323		; Src A Mod LSB
	sta	$d333		; Dest Mod LSB
	sta	$d33b		; Disable Src B
	lda	#$10
	sta	$d327		; Src A Frac
	sta	$d337		; Dest Frac
	lda	#%00011000
	sta	$d33e		; minterms + shift
	endm

;**************************************************************************
;*
;* NAME  find_file_in_dir
;*
;* DESCRIPTION
;*   find file in cached directory
;*
;*   Sets up track/sector/offset.
;*   C=1 means not found.
;*
;******
find_file_in_dir:
	lda	#2
	sta	lptr_zp
	lda	#>dir_buffer
	sta	lptr_zp+1
	ldx	dir_cnt
	beq	ff_fl1
ff_lp1:
	ldy	#0
ff_lp2:
	lda	(namptr_zp),y
	cmp	#"*"
	beq	ff_ex1
	cmp	(lptr_zp),y
	bne	ff_skp2
	iny
	cpy	namlen_zp
	bne	ff_lp2
	beq	ff_ex1
ff_skp2:
	clc
	lda	#DIR_ENTRY_LEN
	adc	lptr_zp
	sta	lptr_zp
	bcc	ff_skp3
	inc	lptr_zp+1
ff_skp3:
	dex
	bne	ff_lp1

ff_fl1:
; file not found
	sec
	rts

; file was found, setup track and sector
ff_ex1:
	dec	lptr_zp+1
	ldy	#$fe
	lda	(lptr_zp),y
	sta	track
	iny
	lda	(lptr_zp),y
	sta	sector
	lda	#2
	sta	offset
	clc
	rts

;**************************************************************************
;*
;* NAME  cache_directory
;*
;* DESCRIPTION
;*   Cache directory contents into the dir_buffer.
;*
;******
cache_directory:
	lda	#18
	jsr	read_track_cached

	ldy	#0
	sty	lptr_zp
	sty	cd_sm1+1
	lda	#>dir_buffer
	sta	cd_sm1+2
	lda	#>track_buffer
	sta	lptr_zp+1
cd_lp1:
	ldy	#0
	lda	(lptr_zp),y
	cmp	#18
	bne	cd_ex1
	iny
	lda	(lptr_zp),y
	clc
	adc	#>track_buffer
	sta	lptr_zp+1
cd_lp2:
	ldy	#2
	lda	(lptr_zp),y
	and	#%10001111
	cmp	#$82		; PRG
	bne	cd_skp2

	ldx	#0
cd_lp3:
	iny
	lda	(lptr_zp),y
cd_sm1:
	sta	dir_buffer,x
	inx
	cpx	#DIR_ENTRY_LEN	; t/s + fname
	bne	cd_lp3
	txa
	clc
	adc	cd_sm1+1
	sta	cd_sm1+1
	bcc	cd_skp1
	inc	cd_sm1+2
cd_skp1:
	inc	dir_cnt
	lda	dir_cnt
	cmp	#MAX_FILES
	beq	cd_ex1
cd_skp2:
	lda	lptr_zp
	clc
	adc	#32
	sta	lptr_zp
	bne	cd_lp2
	beq	cd_lp1
cd_ex1:
	rts

;**************************************************************************
;*
;* NAME  read_track_cached
;*
;* DESCRIPTION
;*   Read track into track buffer unless it's already there.
;*   Retry forever on errors.
;*
;******
read_track_cached:
	cmp	last_track
	beq	rtc_ex1
	sta	last_track
	sta	track_zp
rtc_lp1:
	jsr	read_track
	bcs	rtc_lp1		; infinite retries
rtc_ex1:
	clc
	rts

; show errors
	if	DEBUG_SECTOR_ERRORS
HAVE_SECTOR_HOOK	equ	1
	mac	SECTOR_HOOK
	cmp	#$01
	beq	.skp
	tax
	inc	$0400+40*24,x
.skp:
	endm
	endif

;**************************************************************************
;*
;* SECTION  external code
;*   
;******
	include	"ld2_dtv.asm"
loader_end:
	SHOWRANGE "loader",loader_start,loader_end
	
; eof
