;**************************************************************************
;*
;* FILE  readdisk2_test.asm
;* Copyright (c) 2005, 2006, 2008 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: readdisk2_test.asm,v 1.79 2009-03-29 16:25:32 tlr Exp $
;*
;* DESCRIPTION
;*   Fast d64 image reader for the DTV.
;*
;*   frequencies: 985250 Hz (PAL), 1022730 (NTSC) and 1000000 MHz (1541)
;*
;*   PAL: 
;*     (985250/1000000)*68 = 66.99700
;*     i.e 67 C64/DTV cycles per 68 drive cycles
;*
;*   NTSC:
;*     (1022730/1000000)*44 = 45.00012 
;*     i.e 45 C64/DTV cycles per 44 drive cycles
;*
;******
	processor 6502

	include	"macros.i"
	INFO	"--- start of file readdisk2_test.asm ---"

HAVE_COUNT_ADDR	equ	0

	include	"ld2_zp_bss.asm"

; Debugging options
	ifnconst DUMP_2BIT
DUMP_2BIT	equ	0
	endif
DUMP_4BIT	equ	0
DUMP_8BIT	equ	0
DUMP_RLE	equ	0
DUMP_BUFFER	equ	$040000
	
HAVE_DUMP	equ	DUMP_2BIT || DUMP_4BIT || DUMP_8BIT || DUMP_RLE
DUMP_PREBUFFER	equ	$a000

BUFFER_START	equ	$020000

	seg.u	zp
;**************************************************************************
;*
;* SECTION  zero page
;*
;******
	org	$20
sptr_zp:
	ds.w	1
dptr_zp
	ds.w	1
dptr2_zp:
	ds.w	1
dtmp_zp:
	ds.b	1

	seg	code
	org	$0801
;**************************************************************************
;*
;* Basic line!
;*
;******
StartOfFile:
	dc.w	EndLine
	dc.w	0
	dc.b	$9e,"2069 /T.L.R/",0
;	        0 SYS2069 /T.L.R/
EndLine:
	dc.w	0
	
;**************************************************************************
;*
;* SysAddress... When run we will enter here!
;*
;******
SysAddress:
; set device number to 8 if it is less than 8
	lda	#8
	cmp	$ba
	bcc	sa_skp1
	sta	$ba
sa_skp1:

; fix screen
	jsr	$ff5b
	lda	#14
	sta	$d020
	sta	646
	lda	#6
	sta	$d021
	jsr	prepare_system

	lda	#<menu_msg
	ldy	#>menu_msg
	jsr	$ab1e
sa_lp1:
	jsr	$ffe4
	cmp	#3
	beq	sa_ex1
	cmp	#13
	bne	sa_lp1

	jsr	clear_sec

	jsr	init_drive
; static setup of buffer segment
	sac	#$dd
	lda	#BUFFER_SEGMENT
	sac	#$00

	jsr	prepare_dma
	if	HAVE_DUMP	
	jsr	prepare_dump
	endif
	jsr	read_disk

	lda	#14
	nopalign 0
sa_lp2:
	sta	$d020
	eor	#1
	bra	sa_lp2
	
sa_ex1:
	jsr	restore_system
	jmp	$a474	


;**************************************************************************
;*
;* NAME  clear_sec, print_sec
;*
;* DESCRIPTION
;*   handle sector display
;*
;******
clear_sec:
	ldx	#0
	lda	#$20
cs_lp1:
	sta	$0400+40*2,x
	sta	$0500+40*2,x
	sta	$0600+40*2,x
	sta	$06e8,x
	inx
	bne	cs_lp1

	lda	#$b0
	sta	digs
	sta	digs+1

	ldx	#0
cs_lp2:
	lda	scrtab_l,x
	sta	sptr_zp
	lda	scrtab_h,x
	sta	sptr_zp+1
	ldy	#0
	lda	#$a0
	sta	(sptr_zp),y
	iny
	lda	digs
	sta	(sptr_zp),y
	iny
	lda	digs+1
	sta	(sptr_zp),y
	iny
	jsr	inc_digs
	tya
	clc
	adc	linelen,x
	tay
	lda	#$a0
cs_lp4:
	sta	(sptr_zp),y
	iny
	cpy	#40
	bne	cs_lp4

	inx
	cpx	#21
	bne	cs_lp2	

	lda	#$a0
	sta	$0400+40*2+0
	sta	$0400+40*2+1
	sta	$0400+40*2+2
	sta	$0400+40*2+38
	sta	$0400+40*2+39
	sta	$0400+40*3+0
	sta	$0400+40*3+1
	sta	$0400+40*3+2
	sta	$0400+40*3+38
	sta	$0400+40*3+39
	
	ldx	#$b0
	stx	digs
	inx
	stx	digs+1
	ldx	#0
cs_lp3:
	lda	digs
	sta	$0400+40*2+3,x
	lda	digs+1
	sta	$0400+40*3+3,x
	jsr	inc_digs
	inx
	cpx	#35
	bne	cs_lp3	

	rts

linelen:
	ds.b	17,35
	dc.b	30
	dc.b	24
	dc.b	17
	dc.b	17

inc_digs:
	inc	digs+1
	lda	digs+1
	cmp	#$ba
	bne	idg_skp1
	lda	#$b0
	sta	digs+1
	inc	digs
idg_skp1:
	rts

digs:
	dc.b	$B0,$B0

; Print entry for track/sector
; Acc = screen code, sector_zp = sector, track_zp = track
print_sec:
	tax
	ldy.w	sector_zp
	lda	scrtab_l,y
	sta	sptr_zp
	lda	scrtab_h,y
	sta	sptr_zp+1
	ldy.w	track_zp
	iny
	iny
	txa
	sta	(sptr_zp),y
	rts

; generate screen position table
BASE	equ	$0400+40*4
scrtab_l:
i	set	0
	repeat	21
	dc.b	<[BASE+i]
i	set	i + 40
	repend
scrtab_h:
i	set	0
	repeat	21
	dc.b	>[BASE+i]
i	set	i + 40
	repend

HAVE_SECTOR_HOOK	equ	1
	mac	SECTOR_HOOK
	bcc	.skp1
	cmp	#$03		; don't print sync lost as
				; sector is invalid then
	beq	.ex1
	clc
	adc	#$30		; error code
	dc.b	$2c
.skp1:
	lda	#"*"
	jsr	print_sec

	if	HAVE_DUMP
	jsr	dump_sector
	endif
.ex1:
	endm

HAVE_SETUP_BLITTER_HOOK	equ	1
	mac	SETUP_BLITTER_HOOK
	if	HAVE_DUMP
	lda	#$20
	sta	$d30b		; DMA length MSB
	endif
	if	[track_buffer_real>>16]
	lda	#track_buffer_real>>16
	sta	$d322		; Src A MSB
	endif
	lda	#[track_buffer_real>>8] & $ff
	sta	$d321		; Src A MidSB
	lda	#BUFFER_START>>16
	sta	$d332		; Dest MSB
	lda	#1
	sta	$d323		; Src A Mod LSB
	sta	$d333		; Dest Mod LSB
	sta	$d33b		; Disable Src B
	lda	#$10
	sta	$d327		; Src A Frac
	sta	$d337		; Dest Frac
	lda	#%00011000
	sta	$d33e		; minterms + shift
; length is setup by the read_disk loop anyway.
; 	lda	#21
; 	sta	$d339		; Blit length MSB
	endm

;**************************************************************************
;*
;* NAME  read_disk
;*
;* DESCRIPTION
;*   read entire disk to memory
;*
;******
read_disk:
	
	lda	#1
	sta.w	track_zp
rd_lp1:
	jsr	read_track

	if	!HAVE_DUMP
	lda.w	num_zp
	sta	$d339
	lda	#%00001111
	sta	$d33a
rd_lp2:
	lda	$d33f
	lsr
	bcs	rd_lp2
	lda	#%00001000	; Dest Continue
	sta	$d33f
	endif

	inc.w	track_zp
	lda.w	track_zp
	cmp	#35+1
	bne	rd_lp1

	lda	#$84		; SEEK
	jsr	put_byte
	lda	#18
	jsr	put_byte

	lda	#$80		; UNINSTALL
	jsr	put_byte
	lda	#14
	sta	$d020
	rts

	if	HAVE_DUMP
;**************************************************************************
;*
;* NAME  prepare_dump, dump_sector
;*
;* DESCRIPTION
;*   Prepare
;*
;******
prepare_dump:
	ldx	#$1f
prd_lp1:
	lda	blittab_dump,x
	sta	$d320,x
	dex
	bpl	prd_lp1

	lda	#>dma_dd00_buffer
	sta	prd_sm1+2
	ldy	#$20
	ldx	#0
	txa
prd_lp2:
prd_sm1:
	sta	dma_dd00_buffer,x		
	inx
	bne	prd_lp2
	inc	prd_sm1+2
	dey
	bne	prd_lp2
	rts

dump_sector:
	sac	#$99
	pha
	lda	#3
	sac	#$00

	if	DUMP_2BIT || DUMP_4BIT
	lda	#<dma_dd00_buffer
	sta	dptr_zp
	lda	#<DUMP_PREBUFFER
	sta	dptr2_zp
	lda	#>dma_dd00_buffer
	sta	dptr_zp+1
	lda	#>DUMP_PREBUFFER
	sta	dptr2_zp+1
	ldx	#$20
	endif
	if	DUMP_2BIT
	nopalign 0
llp3:
	ldy	#0
llp2:
	lda	(dptr_zp),y
	nop
	rol
	rol	dtmp_zp
	iny
	rol
	rol	dtmp_zp
	cpy	#4
	bne	llp2
	ldy	#0
	lda	dtmp_zp
	nop
	nop
	sta	(dptr2_zp),y
	nop
	nop
	inc	dptr2_zp
	bne	skp2
	inc	dptr2_zp+1
skp2:
	nop
	clc
	lda	dptr_zp
	adc	#4
	sta	dptr_zp
	bcc	llp3
	inc	dptr_zp+1
	nop
	dex
	bne	llp3
	endif	;DUMP_2BIT
	if	DUMP_4BIT
	nopalign 0
llp3:
	ldy	#0
	lda	(dptr_zp),y
	nop
	sec
	sbc	#$07
	tay
	rol
	rol	dtmp_zp
	nop
	rol
	rol	dtmp_zp
	nop
	nop
	nop
	tya
	lsr
	lsr
	rol	dtmp_zp
	tya
	lsr
	rol	dtmp_zp

	ldy	#1
	lda	(dptr_zp),y
	nop
	sec
	sbc	#$07
	tay
	rol
	rol	dtmp_zp
	nop
	rol
	rol	dtmp_zp
	nop
	nop
	nop
	tya
	lsr
	lsr
	rol	dtmp_zp
	tya
	lsr
	rol	dtmp_zp
	
	ldy	#0
	lda	dtmp_zp
	nop
	nop
	sta	(dptr2_zp),y
	nop
	nop
	inc	dptr2_zp
	bne	skp2
	inc	dptr2_zp+1
skp2:
	nop
	clc
	lda	dptr_zp
	adc	#2
	sta	dptr_zp
	bcc	llp3
	inc	dptr_zp+1
	nop
	dex
	bne	llp3
	endif	;DUMP_4BIT

	lda	#%00001111
	sta	$d33a
llp:
	lda	$d33f
	lsr
	bcs	llp

; Src A Continue
	lda	#%00001000
	sta	$d33f

	sac	#$99
	pla
	sac	#$00
	rts
	
blittab_dump:
	if	DUMP_8BIT
	dc_l	dma_dd00_buffer	; $d320 src A
	else
	dc_l	DUMP_PREBUFFER	; $d320 src A
	endif
	dc.w	1		; $d323 mod A
	dc.w	0		; $d325 line len A
	dc.b	$10		; $d327 frac A
	dc.b	0,0,0		; $d328 src B
	dc.w	0		; $d32b mod B
	dc.w	0		; $d32d line len B
	dc.b	0		; $d32f frac B
	dc_l	DUMP_BUFFER	; $d330 dest D
	dc.w	1		; $d333 mod D
	dc.w	0		; $d335 line len D
	dc.b	$10		; $d337 frac D
	if	DUMP_8BIT
	dc.w	$2000		; $d338 blit length
	endif
	if	DUMP_2BIT
	dc.w	$0800		; $d338 blit length
	endif
	dc.b	0		; $d33a start
	dc.b	%00000001	; $d33b disable B
	dc.b	0,0		; not used
	dc.b	%00011000	; $d33e minterms + shift
	dc.b	%00000000	; $d33f continues
	endif

	
;**************************************************************************
;*
;* NAME  prepare_system
;*
;* DESCRIPTION
;*   Prepare
;*
;******
prepare_system:	
; enable extended registers
	lda	#1
	sta	$d03f

; turn off badlines and color fetch
	lda	#%00110000
	sta	$d03c

; replace black with the desired forground
	ldx	646
	lda	$fa56,x
	sta	$d200
	stx	text_color_tmp
	lda	#0
	sta	646
	rts

text_color_tmp:
	ds.b	1

;**************************************************************************
;*
;* NAME  restore_system
;*
;* DESCRIPTION
;*   Restore
;*
;******
restore_system:
; restore text color and palette
	lda	text_color_tmp
	sta	646
	ldx	#$10
rs_lp1:
	lda	$fa56-1,x
	sta	$d200-1,x
	dex
	bne	rs_lp1

; 	ldx	#%00000000	
	stx	$d03c		; reenable badlines + color fetch
; 	ldx	#0
	stx	$d03f		; exit extended mode
	rts

menu_msg:
	dc.b	147,"DTV2/3 DISK READER MK II / TLR",13
	dc.b	"----------------------------------------",13
	if	!HAVE_DUMP
	dc.b	"READS DISK TO D64-IMAGE AT $020000.",13
	else
	dc.b	"READS RAW $DD00 BITS TO $040000-$195800."
	endif
	dc.b	13
	dc.b	"INSERT DISK AND PRESS ENTER...",13
	dc.b	0

;**************************************************************************
;*
;* SECTION  external code
;*   
;******
	include	"ld2_dtv.asm"
	align	256
	include	"ld2_install.asm"
	
; eof
